import unittest

class TestCountryData(unittest.TestCase):

    def test_load_country_us(self):
        from format_currency import Country
        country = Country.load_country('US')
        self.assertIsNotNone(country)
        self.assertEqual(country.name, 'United States')
        self.assertEqual(country.alpha2, 'US')
        self.assertEqual(country.alpha3, 'USA')
        self.assertEqual(country.currency_code, 'USD')
        self.assertEqual(country.currency_name, 'Dollar')
        self.assertEqual(country.currency_symbol, '$')
        self.assertEqual(country.currency_decimal_separator, '.')
        self.assertEqual(country.currency_thousands_separator, ',')
        self.assertEqual(country.currency_decimal_place, 2)

    def test_load_country_us_lowercase(self):
        from format_currency import Country
        country = Country.load_country('us')
        self.assertIsNotNone(country)
        self.assertEqual(country.name, 'United States')
        self.assertEqual(country.alpha2, 'US')
        self.assertEqual(country.alpha3, 'USA')
        self.assertEqual(country.currency_code, 'USD')
        self.assertEqual(country.currency_name, 'Dollar')
        self.assertEqual(country.currency_symbol, '$')
        self.assertEqual(country.currency_decimal_separator, '.')
        self.assertEqual(country.currency_thousands_separator, ',')
        self.assertEqual(country.currency_decimal_place, 2)

    def test_load_country_us_cached(self):
        from format_currency import Country
        country = Country.load_country('US')
        self.assertIsNotNone(country)
        self.assertEqual(country.name, 'United States')
        self.assertEqual(country.alpha2, 'US')
        self.assertEqual(country.alpha3, 'USA')
        self.assertEqual(country.currency_code, 'USD')
        self.assertEqual(country.currency_name, 'Dollar')
        self.assertEqual(country.currency_symbol, '$')
        self.assertEqual(country.currency_decimal_place, 2)

    def test_load_country_id(self):
        from format_currency import Country
        country = Country.load_country('ID')
        self.assertIsNotNone(country)
        self.assertEqual(country.name, 'Indonesia')
        self.assertEqual(country.alpha2, 'ID')
        self.assertEqual(country.alpha3, 'IDN')
        self.assertEqual(country.currency_code, 'IDR')
        self.assertEqual(country.currency_name, 'Rupiah')
        self.assertEqual(country.currency_symbol, 'Rp')
        self.assertEqual(country.currency_decimal_separator, ',')
        self.assertEqual(country.currency_thousands_separator, '.')
        self.assertEqual(country.currency_decimal_place, 2)

    def test_load_country_by_currency_code_usd(self):
        from format_currency import Country
        country = Country.load_country_by_currency_code('USD')
        self.assertIsNotNone(country)
        self.assertEqual(country.name, 'United States')
        self.assertEqual(country.alpha2, 'US')
        self.assertEqual(country.alpha3, 'USA')
        self.assertEqual(country.currency_code, 'USD')
        self.assertEqual(country.currency_name, 'Dollar')
        self.assertEqual(country.currency_symbol, '$')
        self.assertEqual(country.currency_decimal_separator, '.')
        self.assertEqual(country.currency_thousands_separator, ',')
        self.assertEqual(country.currency_decimal_place, 2)

    def test_load_country_by_currency_code_usd_lowercase(self):
        from format_currency import Country
        country = Country.load_country_by_currency_code('usd')
        self.assertIsNotNone(country)
        self.assertEqual(country.name, 'United States')
        self.assertEqual(country.alpha2, 'US')
        self.assertEqual(country.alpha3, 'USA')
        self.assertEqual(country.currency_code, 'USD')
        self.assertEqual(country.currency_name, 'Dollar')
        self.assertEqual(country.currency_symbol, '$')
        self.assertEqual(country.currency_decimal_separator, '.')
        self.assertEqual(country.currency_thousands_separator, ',')
        self.assertEqual(country.currency_decimal_place, 2)

    def test_load_country_bh(self):
        from format_currency import Country
        country = Country.load_country('BH')
        self.assertIsNotNone(country)
        self.assertEqual(country.name, 'Bahrain')
        self.assertEqual(country.alpha2, 'BH')
        self.assertEqual(country.alpha3, 'BHR')
        self.assertEqual(country.currency_code, 'BHD')
        self.assertEqual(country.currency_name, 'Dinar')
        self.assertEqual(country.currency_symbol, None)
        self.assertEqual(country.currency_decimal_separator, '.')
        self.assertEqual(country.currency_thousands_separator, ',')
        self.assertEqual(country.currency_decimal_place, 3)

    def test_load_country_not_found(self):
        from format_currency import Country
        country = Country.load_country('XX')
        self.assertIsNone(country)



import locale

from .country import Country

def format_currency(number, country_code=None, currency_code=None, currency_symbol=None, decimal_separator=None, thousands_separator=None, decimal_places=None, use_current_locale=False, **kwargs):
    """
    Formats a given number as a currency string according to various parameters and optional locale settings.
    
    Parameters:
    - number (float): The numerical value to be formatted.
    - country_code (str, optional): The country code to determine currency formatting rules. Defaults to None.
    - currency_code (str, optional): The currency code to determine currency formatting rules. Defaults to None.
    - currency_symbol (str, optional): The symbol to be used for the currency. Defaults to None.
    - decimal_separator (str, optional): The character to use as a decimal separator. Defaults to None.
    - thousands_separator (str, optional): The character to use as a thousands separator. Defaults to None.
    - decimal_places (int, optional): The number of decimal places to display. Defaults to None.
    - use_current_locale (bool, optional): Whether to use the current locale settings for formatting. Defaults to False.
    - kwargs (dict): Additional keyword arguments.
    
    Allowed kwargs:
    - place_currency_symbol_at_end (bool): Whether to place the currency symbol at the end of the formatted number. Default is false.
    
    Returns:
    - str: The formatted currency string.
    
    Raises:
    - TypeError: If unexpected keyword arguments are provided.
    
    Notes:
    - If no formatting options are provided, auto-formatting is determined based on the country or currency code.
    - Supports special numbering systems for India and China if the country code is 'IN', 'BD', 'NP', 'PK', or 'CN' respectively.
    """
    allowed_kwargs = ['place_currency_symbol_at_end']
    __unexpected_args_provided = []
    for kwarg in kwargs:
        if kwarg not in allowed_kwargs:
            __unexpected_args_provided.append(kwarg)
    if len(__unexpected_args_provided) > 0:
        raise TypeError(f"Unexpected keyword argument(s) '{__unexpected_args_provided}'\nAllowed kwargs are {allowed_kwargs}.")
    
    # Get the value(s) of kwargs
    place_currency_symbol_at_end = kwargs.get('place_currency_symbol_at_end', False)

    autoformat = True
    indian_numbering_system = False
    china_numbering_system = False

    if currency_symbol or decimal_separator or thousands_separator:
        autoformat = False

    if autoformat:
        country = None
        currency_symbol = ''
        decimal_separator = '.'
        thousands_separator = ','
        decimal_places = 2

        if country_code:
            country = Country.load_country(country_code)
        if currency_code:
            country = Country.load_country_by_currency_code(currency_code)

        if country:
            if country.currency_symbol:
                currency_symbol = country.currency_symbol
            else:
                currency_symbol = country.currency_code

            country_code = country.alpha2
            currency_code = country.currency_code
            decimal_separator = country.currency_decimal_separator
            thousands_separator = country.currency_thousands_separator
            decimal_places = country.currency_decimal_place

    if country_code in ('IN', 'BD', 'NP', 'PK'):
        indian_numbering_system = True
 
    if country_code == 'CN':
        china_numbering_system = True
 
    if use_current_locale:
        localeconv = locale.localeconv()
        thousands_separator = localeconv.get('mon_thousands_sep', thousands_separator)
        decimal_separator = localeconv.get('mon_decimal_point', decimal_separator)

    formatting_precision = f'.0{decimal_places}f'
    formatting = '{:,' + formatting_precision + '}'

    # return formatting
    formatted_number = formatting.format(number)

    if (not use_current_locale) and indian_numbering_system:
        formatted_number = format_india_numbering_system(formatted_number)
    if (not use_current_locale) and china_numbering_system:
        formatted_number = format_china_numbering_system(formatted_number)

    formatted_number = formatted_number.replace('.', '_')
    formatted_number = formatted_number.replace(',', thousands_separator)
    formatted_number = formatted_number.replace('_', decimal_separator)
    if place_currency_symbol_at_end:
        return f'{formatted_number} {currency_symbol}'.strip()
    return f'{currency_symbol} {formatted_number}'.strip()


def format_india_numbering_system(number_str):
    is_negative = number_str.startswith('-')

    if is_negative:
        number_str = number_str[1:]

    number_str = number_str.replace(',', '')
    s, *d = number_str.partition(".")
    r = ",".join([s[x-2:x] for x in range(-3, -len(s), -2)][::-1] + [s[-3:]])
    result = "".join([r] + d)

    if is_negative:
        return '-' + result

    return result


def format_china_numbering_system(number_str):
    is_negative = number_str.startswith('-')

    if is_negative:
        number_str = number_str[1:]

    number_str = number_str.replace(',', '')
    s, *d = number_str.partition(".")
    r = ",".join([s[x-4:x] for x in range(-4, -len(s), -4)][::-1] + [s[-4:]])
    result = "".join([r] + d)

    if is_negative:
        return '-' + result

    return result



import unittest

class TestCurrencyFormat(unittest.TestCase):

    def test_currency_format_us(self):
        from format_currency import format_currency
        self.assertEqual(format_currency(1234567.89, 'US'), '$ 1,234,567.89')
        self.assertEqual(format_currency(1234567.89, currency_code='USD'), '$ 1,234,567.89')
        self.assertEqual(format_currency(1234567.89, 'us'), '$ 1,234,567.89')
        self.assertEqual(format_currency(1234567.89, currency_code='usd'), '$ 1,234,567.89')

        import locale
        locale.setlocale(locale.LC_ALL, '')
        self.assertEqual(format_currency(1234567.89, 'US', use_current_locale=True), '$ 1,234,567.89')
        self.assertEqual(format_currency(1234567.89, currency_code='USD', use_current_locale=True), '$ 1,234,567.89')

    def test_currency_format_id(self):
        from format_currency import format_currency
        self.assertEqual(format_currency(1234567.89, 'ID'), 'Rp 1.234.567,89')
        self.assertEqual(format_currency(-1234567.89, 'ID'), 'Rp -1.234.567,89')
        self.assertEqual(format_currency(1234567.89, currency_code='IDR'), 'Rp 1.234.567,89')
        self.assertEqual(format_currency(1234567.89, 'id'), 'Rp 1.234.567,89')
        self.assertEqual(format_currency(1234567.89, currency_code='idr'), 'Rp 1.234.567,89')

        import locale
        locale.setlocale(locale.LC_ALL, '')
        self.assertEqual(format_currency(1234567.89, 'ID', use_current_locale=True), 'Rp 1,234,567.89')
        self.assertEqual(format_currency(1234567.89, currency_code='IDR', use_current_locale=True), 'Rp 1,234,567.89')

    def test_currency_format_bh(self):
        from format_currency import format_currency
        self.assertEqual(format_currency(1234567.890, 'BH'), 'BHD 1,234,567.890')

        import locale
        locale.setlocale(locale.LC_ALL, '')
        self.assertEqual(format_currency(1234567.890, 'BH', use_current_locale=True), 'BHD 1,234,567.890')

    def test_currency_format_bi(self):
        from format_currency import format_currency
        self.assertEqual(format_currency(1234567.00, 'BI'), 'BIF 1,234,567')

        import locale
        locale.setlocale(locale.LC_ALL, '')
        self.assertEqual(format_currency(1234567.00, 'BI', use_current_locale=True), 'BIF 1,234,567')

    def test_currency_format_in(self):
        import locale
        locale.setlocale(locale.LC_ALL, '')
        from format_currency import format_currency

        self.assertEqual(format_currency(1234567.891, 'IN'), '₹ 12,34,567.89')
        self.assertEqual(format_currency(-1234567.891, 'IN'), '₹ -12,34,567.89')
        self.assertEqual(format_currency(1234567.891, currency_code='INR'), '₹ 12,34,567.89')

        self.assertEqual(format_currency(1234567.891, 'IN', use_current_locale=True), '₹ 1,234,567.89')
        self.assertEqual(format_currency(1234567.891, currency_code='INR', use_current_locale=True), '₹ 1,234,567.89')

    def test_currency_format_cn(self):
        import locale
        locale.setlocale(locale.LC_ALL, '')
        from format_currency import format_currency

        self.assertEqual(format_currency(1234567.891, 'CN'), '¥ 123,4567.89')
        self.assertEqual(format_currency(123456789.123, 'CN'), '¥ 1,2345,6789.12')
        self.assertEqual(format_currency(12345678912345.123, 'CN'), '¥ 12,3456,7891,2345.12')
        self.assertEqual(format_currency(-1234567.891, 'CN'), '¥ -123,4567.89')
        self.assertEqual(format_currency(1234567.891, currency_code='CNY'), '¥ 123,4567.89')

        self.assertEqual(format_currency(1234567.891, 'CN', use_current_locale=True), '¥ 1,234,567.89')
        self.assertEqual(format_currency(1234567.891, currency_code='CNY', use_current_locale=True), '¥ 1,234,567.89')

    def test_load_country_not_found(self):
        from format_currency import format_currency
        self.assertEqual(format_currency(1234567.89, 'XX'), '1,234,567.89')



import os, json

cached_countries_data = None
cached_countries_data_dict = {}
cached_countries_data_by_currency_code_dict = {}

class Country():
    """Country Data"""
    def __init__(self, name, alpha2, alpha3, currency_code, currency_name, currency_symbol, currency_decimal_separator, currency_thousands_separator, currency_decimal_place=2, flag_base64=None):
        super().__init__()
        self.name = name
        self.alpha2 = alpha2
        self.alpha3 = alpha3
        self.currency_code = currency_code
        self.currency_name = currency_name
        self.currency_symbol = currency_symbol
        self.currency_decimal_place = currency_decimal_place
        self.currency_decimal_separator = currency_decimal_separator
        self.currency_thousands_separator = currency_thousands_separator
        self.flag_base64 = flag_base64

    @classmethod
    def load_countries_data(cls):
        """Load countries data from json"""
        global cached_countries_data

        if cached_countries_data is not None:
            return cached_countries_data

        data_json_path = os.path.join(os.path.dirname(__file__), 'data', 'countries.json')
        data = None
        try:
            with open(data_json_path) as f:
                data = json.load(f)
        except UnicodeDecodeError as e:
            with open(data_json_path, encoding = 'utf-8') as f:
                data = json.load(f)

        cached_countries_data = data
        return data

    @classmethod
    def load_country(cls, country_code):
        """Find country by it's ISO code"""
        global cached_countries_data_dict
        cached = cached_countries_data_dict.get(country_code, None)
        if cached is not None:
            return cached

        data = cls.load_countries_data()

        for country_data in data:
            if country_data.get('isoAlpha2', None) == country_code.upper():
                currency_data = country_data.get('currency', {})

                country = cls(
                            name=country_data.get('name', ''),
                            alpha2=country_data.get('isoAlpha2', ''),
                            alpha3=country_data.get('isoAlpha3', ''),
                            flag_base64=country_data.get('flag', None),
                            currency_code=currency_data.get('code', ''),
                            currency_name=currency_data.get('name', ''),
                            currency_symbol=currency_data.get('symbol', ''),
                            currency_decimal_separator=currency_data.get('decimal', '.'),
                            currency_thousands_separator=currency_data.get('thousands', ','),
                            currency_decimal_place=currency_data.get('decimalPlaces', 2),
                           )
                cached_countries_data_dict[country_code] = country
                return country
        
    @classmethod
    def load_country_by_currency_code(cls, currency_code):
        """Find country by it's currency code"""
        global cached_countries_data_by_currency_code_dict
        cached = cached_countries_data_by_currency_code_dict.get(currency_code, None)
        if cached is not None:
            return cached

        data = cls.load_countries_data()

        for country_data in data:
            currency_data = country_data.get('currency')
            if currency_data and currency_data.get('code', None) == currency_code.upper():
                currency_data = country_data.get('currency', {})

                country = cls(
                            name=country_data.get('name', ''),
                            alpha2=country_data.get('isoAlpha2', ''),
                            alpha3=country_data.get('isoAlpha3', ''),
                            flag_base64=country_data.get('flag', None),
                            currency_code=currency_data.get('code', ''),
                            currency_name=currency_data.get('name', ''),
                            currency_symbol=currency_data.get('symbol', ''),
                            currency_decimal_separator=currency_data.get('decimal', '.'),
                            currency_thousands_separator=currency_data.get('thousands', ','),
                            currency_decimal_place=currency_data.get('decimalPlaces', 2),
                           )
                cached_countries_data_by_currency_code_dict[currency_code] = country
                return country
        